# -*- encoding: utf-8 -*-
#
# Pin, una biblioteca para crear aplicaciones
# gráficas muy rápidamente.
#
import sys
import inspect

from PyQt4 import QtGui
from PyQt4 import QtCore
from PyQt4 import uic

import mensajes

# Atrapa la señar CTRL+C
import signal
signal.signal(signal.SIGINT, signal.SIG_DFL)

# hace global el estado de la aplicación.
app = None

def iniciar():
    global app
    app = QtGui.QApplication(sys.argv)

def ejecutar():
    global app
    sys.exit(app.exec_())


class Base(object):

    def __init__(self, ui, centrar=True, mostrar=True):
        self.view = uic.loadUi(ui)

        if centrar:
            self.centrar()

        if mostrar:
            self.view.show()

        self._conectar_senales_automaticamente()

    def obtener_widgets_con_nombre(self):
        """Retorna un diccionario con los nombres y referencias a los widgets.

        Por ejemplo, dado un widget:

            >>> ventana.obtener_widgets_con_nombre()
            {
                'splitter': <PyQt4.QtGui.QSplitter object at 0x90ccbfc>, 
                'centralwidget': <PyQt4.QtGui.QWidget object at 0x90cca04>,
            }

        """

        hijos = self.obtener_hijos(self.view)
        widgets = {}

        for h in hijos:
            if h.objectName():
                widgets[str(h.objectName())] = h

        return widgets

    def obtener_hijos(self, widget):
        """Obtiene los widgets hijos en forma de generador."""
        hijos = widget.children()

        for hijo in hijos:
            yield hijo
            for x in self.obtener_hijos(hijo):
                yield x

    def obtener_senales(self, widget):
        """Retorna una lista de todas las señales que entiende un widget."""
        for name in dir(widget):
            obj = getattr(widget, name)
            if inspect.isclass(obj) and issubclass(obj, QtCore.QObject):
                for name2 in dir(obj):
                    obj2 = getattr(obj, name2)

                    if isinstance(obj2, QtCore.pyqtSignal):
                        yield name, name2

    def _conectar_senales_automaticamente(self):
        """Se encarga de conectar senales a metodos usando nombres.

        Para que un metodo se conecte a una senal de un widget se
        debe escribir el nombre de metodo de una manera especial, el
        nombre de metodo debe comenzar con "on_", luego tener
        el nombre del widget y por ultimo dos guiones y el nombre
        de la senal.

        Por ejemplo, estos son algunos nombres validos::

            - on_close__clicked(self, senal):
            - on_mainWindow__destroyed(self, signal):
        """

        # recolecta todos los metodos que parezcan definir slots.
        candidatos = [metodo for metodo in dir(self) if metodo.startswith('on_')]
        widgets = self.obtener_widgets_con_nombre()

        for metodo in candidatos:
            widget, senal = metodo[3:].split("__")

            try:
                senal_obj = getattr(widgets[widget], senal)
                senal_obj.connect(getattr(self, metodo))
            except KeyError:
                print "Error, no se encuentra el widget de nombre:", widget
            except AttributeError:
                print "El widget '%s' no tiene una senal de nombre '%s'" %(widget, senal)

    def centrar(self):
        """Busca colocar la ventana principal en el centro del escritorio."""
        screen = QtGui.QDesktopWidget().screenGeometry()
        size =  self.view.geometry()
        self.view.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)

    def listar_widgets_y_senales(self):
        """Imprime en pantalla todos los widgets y sus respectivas señales."""
        widgets = self.obtener_widgets_con_nombre()

        for widget in widgets:
            print widget
            for s in self.obtener_senales(widgets[widget]):
                print "\t", s[1]


    def reemplazar(self, widget_anterior, widget_nuevo, fila=0, columna=0, filas_ocupa=1, columnas_ocupa=1):
        parent = widget_anterior.parent()
        parent.layout().addWidget(widget_nuevo, fila, columna, filas_ocupa, columnas_ocupa)
        widget_anterior.close()



# -*- coding: utf-8 *-*
from PyQt4.QtGui import QCompleter
from PyQt4.QtGui import QListWidgetItem
from PyQt4.QtCore import Qt
from PyQt4.QtGui import QListWidget
#Based in pycomplete emacs module.

import sys
import types
import inspect
import StringIO
import os

_HELPOUT = StringIO.StringIO
_STDOUT = sys.stdout

try:
    x = set
except NameError:
    from sets import Set as set
else:
    del x

COLWIDTH = 20


def complete(s, fname=None, imports=None, debug=False):
    '''Display completion in Emacs window'''

    if not s:
        return ''
    # changes to where the file is
    if fname:
        os.chdir(os.path.dirname(fname))

    completions = get_all_completions(s, imports)
    completions.sort(key=lambda x: len(x), reverse=True)

    dots = s.split('.')
    result = os.path.commonprefix([k[len(dots[-1]):] for k in completions])

    if result == '' or result not in completions:
        if completions:
            width = 80

            column = width / COLWIDTH
            white = ' ' * COLWIDTH
            msg = ''

            counter = 0
            for completion in completions:
                if len(completion) < COLWIDTH:
                    msg += completion + white[len(completion):]
                    counter += 1
                else:
                    msg += completion + white[len(completion) - COLWIDTH:]
                    counter += 2

                if counter >= column:
                    counter = 0
                    msg += '\n'
        else:
            msg = 'no completions!'
        if debug:
            return set(completions)

    return result


def get_signature(s, fname=None):
    '''Return info about function parameters'''

    if not s:
        return ''
    # changes to where the file is
    if fname:
        os.chdir(os.path.dirname(fname))

    obj = None
    sig = ""

    try:
        obj = _load_symbol(s, globals(), locals())
    except Exception, ex:
        return '%s' % ex

    if type(obj) in (types.ClassType, types.TypeType):
        # Look for the highest __init__ in the class chain.
        obj = _find_constructor(obj)
    elif type(obj) == types.MethodType:
        # bit of a hack for methods - turn it into a function
        # but we drop the "self" param.
        obj = obj.im_func

    if type(obj) in [types.FunctionType, types.LambdaType]:
        (args, varargs, varkw, defaults) = inspect.getargspec(obj)
        sig = ('%s: %s' % (obj.__name__,
                           inspect.formatargspec(args, varargs, varkw,
                                                 defaults)))
    doc = getattr(obj, '__doc__', '')
    if doc and not sig:
        doc = doc.lstrip()
        pos = doc.find('\n')
        if pos < 0 or pos > 70:
            pos = 70
        sig = doc[:pos]
    return sig


def _load_symbol(s, dglobals, dlocals):
    sym = None
    dots = s.split('.')
    if not s or len(dots) == 1:
        sym = eval(s, dglobals, dlocals)
    else:
        for i in range(1, len(dots) + 1):
            s = '.'.join(dots[:i])
            if not s:
                continue
            try:
                sym = eval(s, dglobals, dlocals)
            except NameError:
                try:
                    sym = __import__(s, dglobals, dlocals, [])
                    dglobals[s] = sym
                except ImportError:
                    pass
            except AttributeError:
                try:
                    sym = __import__(s, dglobals, dlocals, [])
                except ImportError:
                    pass
    return sym


def _import_modules(imports, dglobals):
    '''If given, execute import statements'''
    if imports is not None:
        for stmt in imports:
            try:
                exec stmt in dglobals
            except TypeError:
                raise TypeError('invalid type: %s' % stmt)
            except Exception:
                continue


def get_all_completions(s, imports=None):
    '''Return contextual completion of s (string of >= zero chars)'''

    dlocals = {}

    _import_modules(imports, globals())

    dots = s.rsplit('.', 1)
    if not s or len(dots) == 1:
        keys = set()
        keys.update(globals().keys())
        keys.update(dlocals.keys())
        import __builtin__
        keys.update(dir(__builtin__))
        keys = list(keys)
        keys.sort()
        if s:
            return [k for k in keys if k.startswith(s)]
        else:
            return keys

    sym = None
    for i in range(1, len(dots)):
        s = '.'.join(dots[:i])
        if not s:
            continue
        try:
            s = unicode(s)
            sym = eval(s, globals(), dlocals)
        except NameError:
            try:
                sym = __import__(s, globals(), dlocals, [])
            except ImportError:
                return []
            except AttributeError:
                try:
                    sym = __import__(s, globals(), dlocals, [])
                except ImportError:
                    pass
    if sym is not None:
        s = dots[-1]
        return [k for k in dir(sym) if k.startswith(s) and \
            (not k.startswith('__') and not k.endswith('__'))]


def _find_constructor(class_ob):
    # Given a class object, return a function object used for the
    # constructor (ie, __init__() ) or None if we can't find one.
    try:
        return class_ob.__init__.im_func
    except AttributeError:
        for base in class_ob.__bases__:
            rc = _find_constructor(base)
            if rc is not None:
                return rc
    return None


class CompleterWidget(QCompleter):

    def __init__(self, editor):
        QCompleter.__init__(self)

        self.setWidget(editor)
        self.popupView = QListWidget()
        self.popupView.setAlternatingRowColors(True)
        self.popupView.setWordWrap(False)
        self.setPopup(self.popupView)
        self.setCompletionMode(QCompleter.PopupCompletion)
        self.setCaseSensitivity(Qt.CaseInsensitive)

    def insert_completion(self):
        insert = self.popupView.currentItem().text()
        extra = insert.length() - self.completionPrefix().length()
        self.widget().textCursor().insertText(insert.right(extra))
        self.popup().hide()

    def complete(self, cr, results):
        results = [x for x in results if 
                        not x.startswith('get_') and
                        not x.startswith('obtener_') and
                        not x.startswith('set_') and
                        not x.startswith('_') and
                        not x.startswith('definir_')]

        self.popupView.clear()
        model = self.obtain_model_items(results)
        self.setModel(model)
        self.popup().setCurrentIndex(model.index(0, 0))
        cr.setWidth(self.popup().sizeHintForColumn(0) \
            + self.popup().verticalScrollBar().sizeHint().width() + 10)
        self.popupView.updateGeometries()
        QCompleter.complete(self, cr)

    def obtain_model_items(self, proposals):
        proposals.sort()
        for p in proposals:
            self.popupView.addItem(QListWidgetItem(p))
        return self.popupView.model()



from PyQt4 import QtGui

def cuidado(mensaje):
    QtGui.QMessageBox.warning(None, "Cuidado", mensaje)

def ingresar(titulo, texto):
    texto, ok = QtGui.QInputDialog.getText(None, titulo, texto)
    return texto



from PyQt4 import QtGui

class Boton(QtGui.QPushButton):

    def __init__(self, parent):
        QtGui.QPushButton.__init__(self, parent)
        self.setText("H")

class Ventana(QtGui.QMainWindow):

    def __init__(self):
        QtGui.QMainWindow.__init__(self)

class Lista(QtGui.QListWidget):

    def __init__(self, parent, elementos=[]):
        QtGui.QListWidget.__init__(self, parent)
        self.definir_elementos(elementos)

    def definir_elementos(self, elementos):
        for e in elementos:
            self.addItem(str(e))
